@@ -47,6 +47,11 @@ typedef enum task_queue_t task_queue_t;
  */
 #define RETRANSMIT_TRIES 5
 
+/**
+ * Maximum jitter in percent.
+ */
+#define RETRANSMIT_JITTER_MAX 20
+
 /**
  * Interval for mobike routability checks in ms.
  */
@@ -297,6 +302,17 @@ struct task_manager_t {
 	void (*destroy) (task_manager_t *this);
 };
 
+/**
+ * Calculate total timeout of the retransmission mechanism.
+ *
+ * This is affected by modifications of retransmit_base, retransmit_timeout,
+ * retransmit_limit or retransmit_tries. The resulting value can then be used
+ * e.g. in kernel plugins to set the system's acquire timeout properly.
+ *
+ * @return					calculated total retransmission timeout in seconds
+ */
+u_int task_manager_total_retransmit_timeout();
+
 /**
  * Create a task manager instance for the correct IKE version.
  *
