@@ -179,7 +179,7 @@ typedef struct {
 } signature_enumerator_t;
 
 METHOD(enumerator_t, enumerate, bool,
-	signature_enumerator_t *this, auth_cfg_t **out)
+	signature_enumerator_t *this, va_list args)
 {
 	signerinfo_t *info;
 	signature_scheme_t scheme;
@@ -187,11 +187,13 @@ METHOD(enumerator_t, enumerate, bool,
 	enumerator_t *enumerator;
 	certificate_t *cert;
 	public_key_t *key;
-	auth_cfg_t *auth;
+	auth_cfg_t *auth, **out;
 	chunk_t chunk, hash, content;
 	hasher_t *hasher;
 	bool valid;
 
+	VA_ARGS_VGET(args, out);
+
 	while (this->inner->enumerate(this->inner, &info))
 	{
 		/* clean up previous round */
@@ -300,7 +302,8 @@ METHOD(container_t, create_signature_enumerator, enumerator_t*,
 
 	INIT(enumerator,
 		.public = {
-			.enumerate = (void*)_enumerate,
+			.enumerate = enumerator_enumerate_default,
+			.venumerate = _enumerate,
 			.destroy = _enumerator_destroy,
 		},
 		.inner = this->signerinfos->create_enumerator(this->signerinfos),
